@@ -2,7 +2,9 @@ use crate::scratch::FilteredScratch;
 use crate::scratch::Scratch;
 use fallible_iterator::FallibleIterator;
 use holo_hash::hash_type::AnyDht;
+use holo_hash::AgentPubKey;
 use holo_hash::AnyDhtHash;
+use holo_hash::DhtOpHash;
 use holo_hash::EntryHash;
 use holo_hash::HeaderHash;
 use holochain_serialized_bytes::prelude::*;
@@ -10,10 +12,15 @@ use holochain_sqlite::rusqlite::named_params;
 use holochain_sqlite::rusqlite::Row;
 use holochain_sqlite::rusqlite::Statement;
 use holochain_sqlite::rusqlite::Transaction;
+use holochain_sqlite::sql::sql_cell::FETCH_OP;
+use holochain_types::dht_op::DhtOp;
+use holochain_types::dht_op::DhtOpHashed;
+use holochain_types::dht_op::DhtOpType;
 use holochain_types::prelude::HasValidationStatus;
 use holochain_types::prelude::Judged;
 use holochain_zome_types::Element;
 use holochain_zome_types::Entry;
+use holochain_zome_types::EntryVisibility;
 use holochain_zome_types::HeaderHashed;
 use holochain_zome_types::SignedHeader;
 use holochain_zome_types::SignedHeaderHashed;
@@ -53,7 +60,6 @@ pub mod prelude {
     pub use super::Txns;
     pub use holochain_sqlite::rusqlite::named_params;
     pub use holochain_sqlite::rusqlite::Row;
-    pub use std::sync::Arc;
 }
 
 /// Alias for the params required by rusqlite query execution
@@ -130,16 +136,48 @@ pub trait Stores<Q: Query> {
     fn get_initial_data(&self, query: Q) -> StateQueryResult<Self::O>;
 }
 
+/// Queries that can have access to private data will
+/// implement this trait.
+pub trait PrivateDataQuery {
+    type Hash;
+
+    /// Construct the query with access to private data for this agent.
+    fn with_private_data_access(hash: Self::Hash, author: Arc<AgentPubKey>) -> Self;
+
+    /// Construct the query without access to private data.
+    fn without_private_data_access(hash: Self::Hash) -> Self;
+}
+
 pub trait Store {
     /// Get an [`Entry`] from this store.
     fn get_entry(&self, hash: &EntryHash) -> StateQueryResult<Option<Entry>>;
 
+    /// Get an [`Entry`] from this store.
+    /// - Will return any public entry.
+    /// - If an author is provided
+    /// and a header for this entry matches
+    /// the author then any entry will be return
+    /// regardless of visibility .
+    fn get_public_or_authored_entry(
+        &self,
+        hash: &EntryHash,
+        author: Option<&AgentPubKey>,
+    ) -> StateQueryResult<Option<Entry>>;
+
     /// Get an [`SignedHeaderHashed`] from this store.
     fn get_header(&self, hash: &HeaderHash) -> StateQueryResult<Option<SignedHeaderHashed>>;
 
     /// Get an [`Element`] from this store.
     fn get_element(&self, hash: &AnyDhtHash) -> StateQueryResult<Option<Element>>;
 
+    /// Get an [`Element`] from this store that is either public or
+    /// authored by the given key.
+    fn get_public_or_authored_element(
+        &self,
+        hash: &AnyDhtHash,
+        author: Option<&AgentPubKey>,
+    ) -> StateQueryResult<Option<Element>>;
+
     /// Check if a hash is contained in the store
     fn contains_hash(&self, hash: &AnyDhtHash) -> StateQueryResult<bool> {
         match *hash.hash_type() {
@@ -202,6 +240,25 @@ impl<'stmt> Store for Txn<'stmt, '_> {
         get_entry_from_db(self.txn, hash)
     }
 
+    fn get_public_or_authored_entry(
+        &self,
+        hash: &EntryHash,
+        author: Option<&AgentPubKey>,
+    ) -> StateQueryResult<Option<Entry>> {
+        // Try to get the entry if it's public.
+        match get_public_entry_from_db(self.txn, hash)? {
+            Some(e) => Ok(Some(e)),
+            None => match author {
+                // If no public entry is found try to find
+                // any authored by this agent.
+                Some(author) => Ok(self
+                    .get_any_authored_element(hash, author)?
+                    .and_then(|el| el.into_inner().1.into_option())),
+                None => Ok(None),
+            },
+        }
+    }
+
     fn contains_entry(&self, hash: &EntryHash) -> StateQueryResult<bool> {
         let exists = self.txn.query_row_named(
             "
@@ -294,6 +351,39 @@ impl<'stmt> Store for Txn<'stmt, '_> {
             AnyDht::Header => self.get_exact_element(&hash.clone().into()),
         }
     }
+
+    fn get_public_or_authored_element(
+        &self,
+        hash: &AnyDhtHash,
+        author: Option<&AgentPubKey>,
+    ) -> StateQueryResult<Option<Element>> {
+        match *hash.hash_type() {
+            // Try to get a public element.
+            AnyDht::Entry => match self.get_any_public_element(&hash.clone().into())? {
+                Some(el) => Ok(Some(el)),
+                None => match author {
+                    // If there are none try to get a private authored element.
+                    Some(author) => self.get_any_authored_element(&hash.clone().into(), author),
+                    // If there are no private authored elements then try to get any element and
+                    // remove the entry.
+                    None => Ok(self
+                        .get_any_element(&hash.clone().into())?
+                        .map(|el| Element::new(el.into_inner().0, None))),
+                },
+            },
+            AnyDht::Header => Ok(self.get_exact_element(&hash.clone().into())?.map(|el| {
+                // Filter out the entry if it's private.
+                let is_private_entry = el.header().entry_type().map_or(false, |et| {
+                    matches!(et.visibility(), EntryVisibility::Private)
+                });
+                if is_private_entry {
+                    Element::new(el.into_inner().0, None)
+                } else {
+                    el
+                }
+            })),
+        }
+    }
 }
 
 impl<'stmt> Txn<'stmt, '_> {
@@ -371,6 +461,91 @@ impl<'stmt> Txn<'stmt, '_> {
             Ok(Some(element??))
         }
     }
+
+    fn get_any_public_element(&self, hash: &EntryHash) -> StateQueryResult<Option<Element>> {
+        let element = self.txn.query_row_named(
+            "
+            SELECT
+            Header.blob AS header_blob, Header.hash, Entry.blob as entry_blob
+            FROM Header
+            JOIN Entry ON Header.entry_hash = Entry.hash
+            WHERE
+            Entry.hash = :hash
+            AND
+            Header.private_entry = 0
+            ",
+            named_params! {
+                ":hash": hash,
+            },
+            |row| {
+                let header =
+                    from_blob::<SignedHeader>(row.get(row.as_ref().column_index("header_blob")?)?);
+                Ok(header.and_then(|header| {
+                    let SignedHeader(header, signature) = header;
+                    let hash: HeaderHash = row.get(row.as_ref().column_index("hash")?)?;
+                    let header = HeaderHashed::with_pre_hashed(header, hash);
+                    let shh = SignedHeaderHashed::with_presigned(header, signature);
+                    let entry: Option<Vec<u8>> =
+                        row.get(row.as_ref().column_index("entry_blob")?)?;
+                    let entry = match entry {
+                        Some(entry) => Some(from_blob::<Entry>(entry)?),
+                        None => None,
+                    };
+                    Ok(Element::new(shh, entry))
+                }))
+            },
+        );
+        if let Err(holochain_sqlite::rusqlite::Error::QueryReturnedNoRows) = &element {
+            Ok(None)
+        } else {
+            Ok(Some(element??))
+        }
+    }
+
+    fn get_any_authored_element(
+        &self,
+        hash: &EntryHash,
+        author: &AgentPubKey,
+    ) -> StateQueryResult<Option<Element>> {
+        let element = self.txn.query_row_named(
+            "
+            SELECT
+            Header.blob AS header_blob, Header.hash, Entry.blob as entry_blob
+            FROM Header
+            JOIN Entry ON Header.entry_hash = Entry.hash
+            WHERE
+            Entry.hash = :hash
+            AND
+            Header.author = :author
+            ",
+            named_params! {
+                ":hash": hash,
+                ":author": author,
+            },
+            |row| {
+                let header =
+                    from_blob::<SignedHeader>(row.get(row.as_ref().column_index("header_blob")?)?);
+                Ok(header.and_then(|header| {
+                    let SignedHeader(header, signature) = header;
+                    let hash: HeaderHash = row.get(row.as_ref().column_index("hash")?)?;
+                    let header = HeaderHashed::with_pre_hashed(header, hash);
+                    let shh = SignedHeaderHashed::with_presigned(header, signature);
+                    let entry: Option<Vec<u8>> =
+                        row.get(row.as_ref().column_index("entry_blob")?)?;
+                    let entry = match entry {
+                        Some(entry) => Some(from_blob::<Entry>(entry)?),
+                        None => None,
+                    };
+                    Ok(Element::new(shh, entry))
+                }))
+            },
+        );
+        if let Err(holochain_sqlite::rusqlite::Error::QueryReturnedNoRows) = &element {
+            Ok(None)
+        } else {
+            Ok(Some(element??))
+        }
+    }
 }
 
 impl<'stmt, Q: Query> StoresIter<Q::Item> for QueryStmt<'stmt, Q> {
@@ -443,6 +618,34 @@ impl<'stmt> Store for Txns<'stmt, '_> {
         }
         Ok(None)
     }
+
+    fn get_public_or_authored_entry(
+        &self,
+        hash: &EntryHash,
+        author: Option<&AgentPubKey>,
+    ) -> StateQueryResult<Option<Entry>> {
+        for txn in &self.txns {
+            let r = txn.get_public_or_authored_entry(hash, author)?;
+            if r.is_some() {
+                return Ok(r);
+            }
+        }
+        Ok(None)
+    }
+
+    fn get_public_or_authored_element(
+        &self,
+        hash: &AnyDhtHash,
+        author: Option<&AgentPubKey>,
+    ) -> StateQueryResult<Option<Element>> {
+        for txn in &self.txns {
+            let r = txn.get_public_or_authored_element(hash, author)?;
+            if r.is_some() {
+                return Ok(r);
+            }
+        }
+        Ok(None)
+    }
 }
 
 impl<'stmt, Q: Query> StoresIter<Q::Item> for QueryStmts<'stmt, Q> {
@@ -512,6 +715,34 @@ impl<'borrow, 'txn> Store for DbScratch<'borrow, 'txn> {
             Ok(r)
         }
     }
+
+    fn get_public_or_authored_entry(
+        &self,
+        hash: &EntryHash,
+        author: Option<&AgentPubKey>,
+    ) -> StateQueryResult<Option<Entry>> {
+        let r = self.txns.get_public_or_authored_entry(hash, author)?;
+        if r.is_none() {
+            // Entries in the scratch are authored by definition.
+            self.scratch.get_entry(hash)
+        } else {
+            Ok(r)
+        }
+    }
+
+    fn get_public_or_authored_element(
+        &self,
+        hash: &AnyDhtHash,
+        author: Option<&AgentPubKey>,
+    ) -> StateQueryResult<Option<Element>> {
+        let r = self.txns.get_public_or_authored_element(hash, author)?;
+        if r.is_none() {
+            // Elements in the scratch are authored by definition.
+            self.scratch.get_element(hash)
+        } else {
+            Ok(r)
+        }
+    }
 }
 
 impl<'stmt, Q> StoresIter<Q::Item> for DbScratchIter<'stmt, Q>
@@ -671,3 +902,85 @@ pub fn get_entry_from_db(
         Ok(Some(entry??))
     }
 }
+
+/// Fetch a public Entry from a DB by its hash.
+pub fn get_public_entry_from_db(
+    txn: &Transaction,
+    entry_hash: &EntryHash,
+) -> StateQueryResult<Option<Entry>> {
+    let entry = txn.query_row_named(
+        "
+        SELECT Entry.blob AS entry_blob FROM Entry
+        JOIN Header ON Header.entry_hash = Entry.hash
+        WHERE Entry.hash = :entry_hash
+        AND
+        Header.private_entry = 0
+        ",
+        named_params! {
+            ":entry_hash": entry_hash,
+        },
+        |row| {
+            Ok(from_blob::<Entry>(
+                row.get(row.as_ref().column_index("entry_blob")?)?,
+            ))
+        },
+    );
+    if let Err(holochain_sqlite::rusqlite::Error::QueryReturnedNoRows) = &entry {
+        Ok(None)
+    } else {
+        Ok(Some(entry??))
+    }
+}
+
+/// Get a [`DhtOp`] from the database
+/// filtering out private entries and
+/// [`DhtOp::StoreEntry`] where the entry
+/// is private.
+/// The ops are suitable for publishing / gossiping.
+pub fn get_public_op_from_db(
+    txn: &Transaction,
+    op_hash: &DhtOpHash,
+) -> StateQueryResult<Option<DhtOpHashed>> {
+    let result = txn.query_row_and_then(
+        FETCH_OP,
+        named_params! {
+            ":hash": op_hash,
+        },
+        |row| {
+            let header = from_blob::<SignedHeader>(row.get("header_blob")?)?;
+            let op_type: DhtOpType = row.get("type")?;
+            if header
+                .0
+                .entry_type()
+                .map_or(false, |et| *et.visibility() == EntryVisibility::Private)
+                && op_type == DhtOpType::StoreEntry
+            {
+                return Ok(None);
+            }
+            let hash: DhtOpHash = row.get("hash")?;
+            // Check the entry isn't private before gossiping it.
+            let mut entry: Option<Entry> = None;
+            if header
+                .0
+                .entry_type()
+                .filter(|et| *et.visibility() == EntryVisibility::Public)
+                .is_some()
+            {
+                let e: Option<Vec<u8>> = row.get("entry_blob")?;
+                entry = match e {
+                    Some(entry) => Some(from_blob::<Entry>(entry)?),
+                    None => None,
+                };
+            }
+            let op = DhtOp::from_type(op_type, header, entry)?;
+            StateQueryResult::Ok(Some(DhtOpHashed::with_pre_hashed(op, hash)))
+        },
+    );
+    match result {
+        Err(StateQueryError::Sql(holochain_sqlite::rusqlite::Error::QueryReturnedNoRows)) => {
+            Ok(None)
+        }
+        Err(e) => Err(e),
+        Ok(result) => Ok(result),
+    }
+}
