@@ -1,4 +1,4 @@
-use crate::prelude::mutations_helpers::insert_valid_authored_op;
+use crate::prelude::mutations_helpers::insert_valid_integrated_op;
 use crate::scratch::Scratch;
 use ::fixt::prelude::*;
 use holo_hash::*;
@@ -50,21 +50,21 @@ async fn get_links() {
     let td = LinkTestData::new();
 
     // - Add link to db.
-    insert_valid_authored_op(&mut txn, td.base_op.clone()).unwrap();
-    insert_valid_authored_op(&mut txn, td.target_op.clone()).unwrap();
-    insert_valid_authored_op(&mut txn, td.create_link_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut txn, td.base_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut txn, td.target_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut txn, td.create_link_op.clone()).unwrap();
 
     // - Check we can get the link query back.
     let r = get_link_query(&mut [&mut txn], None, td.tag_query.clone());
     assert_eq!(r[0], td.link);
 
     // - Add the same link to the cache.
-    insert_valid_authored_op(&mut cache_txn, td.base_op.clone()).unwrap();
-    insert_valid_authored_op(&mut cache_txn, td.target_op.clone()).unwrap();
-    insert_valid_authored_op(&mut cache_txn, td.create_link_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut cache_txn, td.base_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut cache_txn, td.target_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut cache_txn, td.create_link_op.clone()).unwrap();
 
     // - Check duplicates don't cause issues.
-    insert_valid_authored_op(&mut cache_txn, td.create_link_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut cache_txn, td.create_link_op.clone()).unwrap();
 
     // - Add to the scratch
     insert_op_scratch(
@@ -102,7 +102,7 @@ async fn get_links() {
     assert_eq!(r.len(), 1);
 
     // - Insert a delete op.
-    insert_valid_authored_op(&mut txn, td.delete_link_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut txn, td.delete_link_op.clone()).unwrap();
 
     let r = get_link_query(
         &mut [&mut cache_txn, &mut txn],
@@ -135,16 +135,16 @@ async fn get_entry() {
     let td = EntryTestData::new();
 
     // - Create an entry on main db.
-    insert_valid_authored_op(&mut txn, td.store_entry_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut txn, td.store_entry_op.clone()).unwrap();
 
     // - Check we get that header back.
     let r = get_entry_query(&mut [&mut txn], None, td.query.clone()).unwrap();
     assert_eq!(*r.entry().as_option().unwrap(), td.entry);
 
     // - Create the same entry in the cache.
-    insert_valid_authored_op(&mut cache_txn, td.store_entry_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut cache_txn, td.store_entry_op.clone()).unwrap();
     // - Check duplicates is ok.
-    insert_valid_authored_op(&mut cache_txn, td.store_entry_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut cache_txn, td.store_entry_op.clone()).unwrap();
 
     // - Add to the scratch
     insert_op_scratch(
@@ -167,7 +167,7 @@ async fn get_entry() {
     assert_eq!(*r.header(), *td.header.header());
 
     // - Delete the entry in the cache.
-    insert_valid_authored_op(&mut cache_txn, td.delete_entry_header_op.clone()).unwrap();
+    insert_valid_integrated_op(&mut cache_txn, td.delete_entry_header_op.clone()).unwrap();
 
     // - Get the entry from both stores and union the queries.
     let r = get_entry_query(
@@ -180,7 +180,7 @@ async fn get_entry() {
     assert!(r.is_none());
 }
 
-/// Test that `insert_valid_authored_op` also inserts a header and potentially an entry
+/// Test that `insert_op` also inserts a header and potentially an entry
 #[tokio::test(flavor = "multi_thread")]
 async fn insert_op_equivalence() {
     observability::test_run().ok();
@@ -225,14 +225,13 @@ async fn insert_op_equivalence() {
         &mut txn1,
         op.to_light(),
         op.as_hash().clone(),
-        true,
         op_order,
         create_header.timestamp,
     )
     .unwrap();
 
     // Insert the op in a single step on conn2
-    insert_valid_authored_op(&mut txn2, op).unwrap();
+    insert_valid_integrated_op(&mut txn2, op).unwrap();
 
     txn1.commit().unwrap();
     txn2.commit().unwrap();
